;;;; srfi-66.scm


(use srfi-4)


(declare 
  (fixnum)
  #;(no-bound-checks)
  (no-procedure-checks)
  (bound-to-procedure
    ##sys#check-structure
    ##sys#check-exact
    ##sys#check-range)
  (export
    u8vector=? u8vector-compare
    u8vector-copy! u8vector-copy) )


(register-feature! 'srfi-66)


(define (u8vector=? u1 u2)
  (##sys#check-structure u1 'u8vector 'u8vector=?)
  (##sys#check-structure u2 'u8vector 'u8vector=?)
  (let* ((u1b (##sys#slot u1 1))
	 (u2b (##sys#slot u2 1))
	 (len (##sys#size u1b)) )
    (and (fx= len (##sys#size u2b))
	 (##core#inline "C_substring_compare" u1b u2b 0 0 len))))

#>!
static ___fixnum u8_compare(___byte_vector b1, ___byte_vector b2, ___fixnum len)
{
  return(C_memcmp(b1, b2, len));
}
<#

(define (u8vector-compare u1 u2)
  (##sys#check-structure u1 'u8vector 'u8vector-compare)
  (##sys#check-structure u2 'u8vector 'u8vector-compare)
  (let* ((u1b (##sys#slot u1 1))
	 (u2b (##sys#slot u2 1))
	 (len1 (##sys#size u1b)) 
	 (len2 (##sys#size u2b)) )
    (cond ((fx< len1 len2) -1)
	  ((fx> len1 len2) 1)
	  (else (u8_compare u1b u2b len1)))))

(define (u8vector-copy! src ss dst ds n)
  (##sys#check-structure src 'u8vector 'u8vector-copy!)
  (##sys#check-structure dst 'u8vector 'u8vector-copy!)
  (##sys#check-exact ss 'u8vector-copy!)
  (##sys#check-exact ds 'u8vector-copy!)
  (##sys#check-exact n 'u8vector-copy!)
  (let* ((sb (##sys#slot src 1))
	 (db (##sys#slot dst 1))
	 (slen (##sys#size sb))
	 (dlen (##sys#size db)) 
	 (sm (fx+ ss n)) )
    (unless (zero? n)
      (##sys#check-range ss 0 slen 'u8vector-copy!)
      (##sys#check-range ds 0 dlen 'u8vector-copy!)
      (##sys#check-range (fx- sm 1) 0 slen 'u8vector-copy!)
      (##core#inline "C_substring_copy" sb db ss sm ds) ) ) )

(define (u8vector-copy u)
  (##sys#check-structure u 'u8vector 'u8vector-copy)
  (let* ((ub (##sys#slot u 1))
	 (len (##sys#size ub)) 
	 (u2 (make-u8vector len)) )
    (##core#inline "C_substring_copy" ub (##sys#slot u2 1) 0 len 0)
    u2) ) 
